unit ProjectInstaller;

interface

uses
  System.ServiceProcess,
  System.Configuration.Install,
  System.ComponentModel,
  System.Collections,
  Microsoft.Win32;

const
  DefaultServiceName = 'ServiceExemple';
  DefaultServiceDisplayName = 'My Windows Service Exemple';
  DefaultServiceDescription = 'This is my Windows service exemple';

type
  [RunInstaller(True)]
  TServiceInstaller = class(System.Configuration.Install.Installer)
  private
    FServicesDependedOn: array of string;
    FServiceProcessInstaller: System.ServiceProcess.ServiceProcessInstaller;
    FServiceInstaller: System.ServiceProcess.ServiceInstaller;

    procedure AddServiceDescription(ServiceDesc: string);
  strict protected
    procedure OnAfterInstall(savedState: IDictionary); override;
  public
    constructor Create;
  end;

var
  ServiceInstaller: TServiceInstaller = nil;

implementation

constructor TServiceInstaller.Create;
begin
  inherited Create;
  SetLength(FServicesDependedOn, 1);
  FServicesDependedOn[0] := 'Event Log';

  FServiceProcessInstaller :=
    System.ServiceProcess.ServiceProcessInstaller.Create;
  FServiceProcessInstaller.Account :=
    System.ServiceProcess.ServiceAccount.LocalSystem;
  FServiceProcessInstaller.Password := '';
  FServiceProcessInstaller.Username := '';

  FServiceInstaller := System.ServiceProcess.ServiceInstaller.Create;
  FServiceInstaller.ServiceName := DefaultServiceName;
  FServiceInstaller.DisplayName := DefaultServiceDisplayName;
  FServiceInstaller.StartType :=
    System.ServiceProcess.ServiceStartMode.Automatic;
  FServiceInstaller.ServicesDependedOn := FServicesDependedOn;

  Installers.Add(FServiceProcessInstaller);
  Installers.Add(FServiceInstaller);
end;

procedure TServiceInstaller.AddServiceDescription(ServiceDesc: string);
var
  ServiceKey: RegistryKey;
begin
  try
    ServiceKey := Microsoft.Win32.Registry.LocalMachine.OpenSubKey(
      'SYSTEM\CurrentControlSet\Services\' + DefaultServiceName, True);
  except
    ServiceKey := nil;
  end;

  if (ServiceKey <> nil) then
  begin
    try
      ServiceKey.SetValue('Description', ServiceDesc);
    except
    end;

    ServiceKey.Close;
  end;
end;

procedure TServiceInstaller.OnAfterInstall(savedState: IDictionary);
begin
  inherited;
  AddServiceDescription(DefaultServiceDescription);
end;

end.
